@@ -1,13 +1,17 @@
 from django.conf import settings
 from django.contrib.sites.models import Site
+from django.contrib import messages
 from django.urls import reverse
 from django.utils.translation import ugettext as _
 from ryzom_django_mdc.html import *  # noqa
 
 
+UNPOLY_TARGET_ALL = '#main, .mdc-top-app-bar__title, #drawer .mdc-list'
+
+
 class A(A):
     attrs = dict(
-        up_target='#main, .mdc-top-app-bar__title, #drawer .mdc-list',
+        up_target=UNPOLY_TARGET_ALL,
         # up_transition='cross-fade',
     )
 
@@ -20,14 +24,22 @@ class Form(Form):
 
 
 class Container(Div):
-    style = {
-        'display': 'flex',
-        'flex-direction': 'column',
-        'align-items': 'center',
-        'max-width': '960px',
-        'width': '96vw',
-        'margin': 'auto',
-    }
+    sass = '''
+    .Container
+        display: flex
+        flex-direction: column
+        align-items: center
+        max-width: 960px
+        width: 96vw
+        margin: auto
+
+    .up-modal .Container
+        width: 86vw
+
+    @media screen and (max-width: 500px)
+        .up-modal .up-modal-content
+            padding: 10px
+    '''
 
 
 class FormContainer(Container):
@@ -36,12 +48,17 @@ class FormContainer(Container):
         max-width: 580px
         .mdc-text-field, .mdc-form-field, .mdc-select, form
             width: 100%
+            max-width: 90vw
     '''
 
 
 class PageMenu(Div):
     attrs = dict(cls='mdc-elevation--z2', style='margin-bottom: 10px')
 
+    def __init__(self, *args, _next=None, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._next = _next
+
     def to_html(self, *content, **context):
         if 'page-menu' not in context:
             return super().to_html(*content, **context)
@@ -53,6 +70,11 @@ class PageMenu(Div):
             if v.urlname == context['view'].urlname:
                 continue
 
+            href = v.url
+            if self._next:
+                if '?' not in href:
+                    href += '?'
+                href += '&_next=' + self._next
             button = A(
                 MDCTextButton(
                     v.label.capitalize(),
@@ -63,7 +85,7 @@ class PageMenu(Div):
                         'color': getattr(v, 'color', 'inherit'),
                     },
                 ),
-                href=v.url,
+                href=href,
                 style='text-decoration: none',
             )
             if getattr(v, 'controller', None) == 'modal':
@@ -84,22 +106,93 @@ class Main(Main):
     pass
 
 
+class ModalClose(Component):
+    '''
+    Close the modal when inserted by Unpoly.
+
+    This should be the default behaviour of Unpoly, as documented, and is the
+    case with modals that do not redirect on the same page, but for modals with
+    a form with a ?_next= redirection then surprisingly unpoly does update the
+    body but does not close the modal by default.
+
+    This tag registers a compiler that calls up.modal.close() when it
+    sees itself, given that unpoly does not execute scripts in
+    response that it loads. As such, this tag MUST be in the initial body, but
+    outside the elements that you will load in modal.
+
+    Remove this hack when we figure the problem in unpoly.
+
+    Equivalent of::
+
+        Script(
+            'up.compiler(".closemodal", function() { up.modal.close(); })',
+            cls='closemodal',
+        ),
+    '''
+    attrs = dict(cls='closemodal')
+
+    def py2js():
+        up.compiler('.closemodal', lambda: up.modal.close())
+
+
+class Message(MDCSnackBar):
+    icons = dict(
+        debug='check_circle',
+        info='check_circle',
+        success='check_circle',
+        warning='warning',
+        error='error',
+    )
+    colors = dict(
+        debug='green',
+        info='yellow',
+        success='green',
+        warning='orange',
+        error='red',
+    )
+    style = '''
+        padding-left: 16px;
+        display: flex;
+        flex-direction: row;
+        align-items: center
+    '''
+
+    def py2js(self):
+        return  # we will use up.compiler
+
+
+class Messages(Div):
+    def to_html(self, *content, **context):
+        msgs = messages.get_messages(context['view'].request)
+        if not msgs:
+            return ''
+
+        return Div(*[
+            Message(message)
+            for message in msgs
+        ]).to_html(*content, **context)
+
+
 class Body(Body):
     style = 'margin: 0'
 
     def __init__(self, *content, **attrs):
         self.drawer = mdcDrawer(id='drawer')
         self.bar = mdcTopAppBar()
+        self.main_inner = Div(
+            Messages(),
+            *content,
+            cls='main-inner',
+        )
         self.main = Main(
-            Div(
-                *content,
-                cls='main-inner',
-            ),
+            self.main_inner,
+            ModalClose(),
             cls='main mdc-drawer-app-content',
             id='main',
         )
         self.debug = settings.DEBUG
         super().__init__(
+            Spinner(),
             self.bar,
             Div(
                 self.drawer,
@@ -109,24 +202,109 @@ class Body(Body):
         )
 
     def py2js(self):
-        up.compiler('[data-mdc-auto-init]', lambda: mdc.autoInit())
+        up.compiler(
+            '[data-mdc-auto-init]',
+            lambda el: mdc.autoInit(el.parentElement)
+        )
         if self.debug:
             up.log.enable()
 
 
+class Spinner(Div):
+    sass = '''
+    .Spinner
+        position: fixed
+        float: right
+        z-index: 20000
+        cursor: pointer
+
+        right: -24px
+        top: 3px
+        @media only screen and (max-width: 600px)
+          top: -32px
+          top: 0
+
+        .lds-dual-ring
+          display: inline-block
+          width: 80px
+          height: 80px
+
+        .lds-dual-ring:after
+          content: " "
+          display: block
+          width: 28px
+          height: 28px
+          margin: 8px
+          border-radius: 50%
+          border: 6px solid #fff
+          border-color: var(--mdc-theme-primary) white var(--mdc-theme-primary) white
+          animation: lds-dual-ring 1.2s linear infinite
+
+        @keyframes lds-dual-ring
+          0%
+            transform: rotate(0deg)
+
+          100%
+            transform: rotate(360deg)
+    '''  # noqa
+
+    def py2js(self):
+        up.proxy.config.slowDelay = 25
+        up.compiler('.Spinner', lambda element: [
+            up.on('up:proxy:slow', lambda: up.element.show(element)),
+            up.on('up:proxy:recover', lambda: up.element.hide(element)),
+        ])
+
+    def __init__(self):
+        super().__init__(Div(cls='lds-dual-ring'), style='display:none')
+
+
+def poll():
+    def poll_setup(element):
+        if not element.getAttribute('id'):
+            console.error('[poll] *REQUIRES* an id attribute!')
+            return
+        interval = parseInt(element.getAttribute('poll') or 5000)
+
+        def poll():
+            if not document.hidden:
+                up.reload(element)
+        poll = setInterval(poll, interval)
+
+        def clear():
+            clearInterval(poll)
+        return clear
+    up.compiler('[poll]', poll_setup)
+
+
+def snack(self):
+    def open_snack(elem):
+        sb = new.mdc.snackbar.MDCSnackbar(elem)
+        sb.open()
+    up.compiler('[data-mdc-auto-init=MDCSnackbar]', open_snack)
+
+
 class App(Html):
     body_class = Body
     scripts = [
-        'https://unpkg.com/unpoly@0.62.1/dist/unpoly.js',
+        'https://unpkg.com/unpoly@1.0.0/dist/unpoly.js',
+        poll,
+        snack,
+        # 'https://unpkg.com/unpoly@2.0.0-rc9/unpoly.min.js',
+        # 'https://unpkg.com/unpoly@2.0.0-rc9/unpoly-migrate.js',
     ]
     stylesheets = [
-        'https://unpkg.com/unpoly@0.62.1/dist/unpoly.min.css',
+        'https://unpkg.com/unpoly@1.0.0/dist/unpoly.css',
+        # 'https://unpkg.com/unpoly@2.0.0-rc9/unpoly.min.css',
     ]
+    sass = '''
+    .up-modal .up-modal-viewport
+        padding-left: 0
+        padding-right: 0
+        padding-top: 10px
+    '''
 
     def to_html(self, *content, **context):
-        self.head.content.append(
-            Style('.up-modal-content { padding-top: 0; padding-bottom: 0; }')
-        )
         if title := getattr(context['view'], 'title', None):
             self.head.content.append(Title(title))
         return super().to_html(*content, **context)
@@ -161,13 +339,24 @@ class FormTemplate(FormContainer):
                 ),
                 href=next_,
             )
+
+        _next = view.request.GET.get(
+            '_next',
+            view.request.POST.get('_next', ''),
+        )
+        if _next:
+            _next = Input(type='hidden', name='_next', value=_next)
+
         return super().to_html(
             H3(view.title),
             Form(
                 form,
                 CSRFInput(view.request),
+                _next,
                 back,
                 MDCButtonRaised(getattr(view, 'title_submit', _('Submit'))),
+                method='post',
+                action=view.request.path_info,
             ),
         )
 
@@ -205,12 +394,19 @@ class Swagger(Div):
         )
 
 
-@template('crudlfap/home.html', App)
+@template('crudlfap/home.html', App, Container)
 class Home(Div):
     def to_html(self, **context):
         site = Site.objects.get_current()
         return super().to_html(
             H1('Welcome to ' + site.name),
+            MDCButtonRaised(
+                'Login to continue',
+                href=reverse('login'),
+                up_target=UNPOLY_TARGET_ALL,
+                tag='a',
+            ),
+            Div('Then, navigate with the menu button at the north west'),
             **context
         )
 
@@ -218,21 +414,20 @@ class Home(Div):
 @template('registration/logged_out.html', App, NarrowCard)
 class LoggedOut(Div):
     def to_html(self, **context):
-        from .site import site
         return super().to_html(
             H1(_('Log out')),
             P(_('Thanks for spending some quality time with the Web site today.')),  # noqa
             A(
                 _('Log in again'),
-                href=site.views['login'].url,
+                href=reverse('login'),
             ),
             **context,
         )
 
 
-@template('crudlfap/detail.html', App, NarrowCard)
+@template('crudlfap/detail.html', App)
 class ObjectDetail(Div):
-    def to_html(self, **context):
+    def to_html(self, *content, **context):
         table = MDCDataTable()
         table.thead.attrs.style.display = 'none'
         table.table.attrs.style.width = '100%'
@@ -244,7 +439,12 @@ class ObjectDetail(Div):
                 MDCDataTableTd(field['value']),
             ))
 
-        return super().to_html(table, **context)
+        return super().to_html(
+            PageMenu(),
+            *content,
+            NarrowCard(table),
+            **context
+        )
 
     def context(self, *content, **context):
         context['page-menu'] = context['view'].router.get_menu(
@@ -290,13 +490,14 @@ class ListActions(Component):
 @template('crudlfap/list.html', App)
 class ObjectList(Div):
     def context(self, *content, **context):
-        context['page-menu'] = context['view'].router.get_menu(
-            'model',
-            context['view'].request,
-        )
+        if 'page-menu' not in context:
+            context['page-menu'] = context['view'].router.get_menu(
+                'model',
+                context['view'].request,
+            )
         return super().context(*content, **context)
 
-    def to_html(self, **context):
+    def to_html(self, *content, **context):
         checkbox = None
         context['listactions'] = context['view'].router.get_menu(
             'list_action',
@@ -335,8 +536,9 @@ class ObjectList(Div):
         table.addchild(self.pagination_component(**context))
 
         return super().to_html(
+            *content,
             self.drawer_component(**context) or '',
-            PageMenu(),
+            PageMenu(_next=context['view'].request.path),
             Div(
                 self.search_component(**context) or '',
                 table,
@@ -708,17 +910,26 @@ class mdcTopAppBar(Header):
     def nav():
         window.drawer.open = not window.drawer.open
 
+    def close():
+        if window.visualViewport.width < 800:
+            window.drawer.open = False
+
     def py2js(self):
-        window.addEventListener('DOMContentLoaded', self.setup)
+        up.compiler('#drawer, #app-bar, #main', self.setup)
 
     def setup():
-        window.drawer = mdc.drawer.MDCDrawer.attachTo(
-            document.getElementById('drawer'))
-        topAppBar = mdc.topAppBar.MDCTopAppBar.attachTo(
-            document.getElementById('app-bar'))
-        topAppBar.setScrollTarget(document.getElementById('main'))
+        drawer = document.getElementById('drawer')
+        app_bar = document.getElementById('app-bar')
+        main = document.getElementById('main')
+
+        window.drawer = mdc.drawer.MDCDrawer.attachTo(drawer)
+        topAppBar = mdc.topAppBar.MDCTopAppBar.attachTo(app_bar)
+        topAppBar.setScrollTarget(main)
         topAppBar.listen('MDCTopAppBar:nav', self.nav)
 
+        for link in drawer.querySelectorAll('a'):
+            link.addEventListener('click', self.close)
+
 
 class mdcDrawer(Aside):
     menu_hooks = []
@@ -744,13 +955,18 @@ class mdcDrawer(Aside):
         for view in site.get_menu('main', request):
             icon = getattr(view, 'icon', None)
             title = getattr(view, 'title', '')
+            attrs = dict()
+            if getattr(view, 'unpoly', True):
+                attrs['up_target'] = UNPOLY_TARGET_ALL
 
             menu_content.append(
-                A(
-                    MDCListItem(title.capitalize(), icon=icon),
+                MDCListItem(
+                    title.capitalize(), icon=icon,
                     href=view.url,
                     style='text-decoration: none',
-                )
+                    tag='a',
+                    **attrs,
+                ),
             )
         for hook in self.menu_hooks:
             menu_content = hook(request, menu_content)
@@ -758,15 +974,16 @@ class mdcDrawer(Aside):
         content = menu_content
 
         if request.session.get('become_user', None):
-            content.append(Li(
+            content.append(
                 A(
                     ' '.join([
                         str(_('Back to your account')),
                         request.session['become_user_realname'],
                     ]),
                     href=reverse('crudlfap:su'),
+                    up_target=UNPOLY_TARGET_ALL,
                 )
-            ))
+            )
 
         return super().to_html(MDCList(*content))
 
