@@ -1,10 +1,12 @@
+import django_filters
 from django import forms
 from django.db import models
 
-import django_filters
-
 
 class FilterMixin(object):
+    def get_filter_model(self):
+        return self.filterset_kwargs['queryset'].model
+
     def get_filterset(self):
         """
         Returns an instance of the filterset to be used in this view.
@@ -15,7 +17,7 @@ class FilterMixin(object):
         # with choices which would empty out results
         for name, field in self.filterset.form.fields.items():
             try:
-                mf = self.model._meta.get_field(name)
+                mf = self.filter_model._meta.get_field(name)
             except Exception:
                 continue
 
@@ -42,7 +44,7 @@ class FilterMixin(object):
     def get_filterset_meta_filter_overrides(self):
         return {
             models.CharField: {
-               'filterset_class': django_filters.CharFilter,
+               'filterset_class': django_filters.CharFilter,  # noqa
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
@@ -66,7 +68,7 @@ class FilterMixin(object):
 
     def get_filterset_meta_attributes(self):
         return dict(
-            model=self.model,
+            model=self.filter_model,
             fields=self.filter_fields,
             filter_overrides=self.filterset_meta_filter_overrides,
             form=self.filterset_form_class,
@@ -79,13 +81,14 @@ class FilterMixin(object):
         extra = dict()
         for field_name in self.filter_fields:
             try:
-                field = self.model._meta.get_field(field_name)
+                field = self.filter_model._meta.get_field(field_name)
             except:  # noqa
                 continue
             choices = getattr(field, 'choices', None)
             if choices is None:
                 continue
-            extra[field_name] = django_filters.ChoiceFilter(choices=choices)
+            extra[field_name] = django_filters.MultipleChoiceFilter(
+                choices=choices)
         return extra
 
     def get_filterset_class_attributes(self):
@@ -95,7 +98,7 @@ class FilterMixin(object):
 
     def get_filterset_class(self):
         return type(
-            '{}FilterSet'.format(self.model.__name__),
+            '{}FilterSet'.format(self.filter_model.__name__),
             (django_filters.FilterSet,),
             self.filterset_class_attributes
         )
