@@ -28,7 +28,7 @@ class ObjectMixin(ModelMixin):
         ])
         if name.lower() == settings.AUTH_USER_MODEL.lower():
             filters |= Q(user_id=self.object.pk)
-        return LogEntry.objects.filter(filters)
+        return LogEntry.objects.filter(filters).order_by('-action_time')
 
     def get_context(self, **context):
         context['object'] = self.object
@@ -158,3 +158,12 @@ class ObjectMixin(ModelMixin):
                 raise
 
         return names
+
+    def get_abstract_object(self):
+        return self.model()
+
+    @classmethod
+    def abstract(cls, **kwargs):
+        if 'object' not in kwargs:
+            kwargs['object'] = cls.abstract_object
+        return cls(**kwargs)
