@@ -49,8 +49,12 @@ dropdown. If there is only one matching view, it will display only the button.
 """
 from django.apps import apps
 from django.conf import settings
+from django.db import models
 from django.urls import path
 from django.utils.module_loading import import_string
+from django.utils.safestring import mark_safe
+
+from crudlfap import html
 
 from .route import Route
 from .settings import CRUDLFAP_VIEWS
@@ -126,6 +130,14 @@ class Router(object):
 
         .. note:: The final views list is generated by the
                   :py:meth:`generate_views` method.
+
+    .. py:attribute:: fields
+
+        Fields that views should use by default.
+
+    .. py:attribute:: json_fields
+
+        Fields that the Router serializer should use by default.
     """
     views = ViewsDescriptor(
         getattr(settings, 'CRUDFLAP_VIEWS', CRUDLFAP_VIEWS)
@@ -329,6 +341,24 @@ class Router(object):
 
         return views
 
+    def get_menu_component(self, name, request, **kwargs):
+        views = self.get_menu(name, request, **kwargs)
+        buttons = []
+        for view in views:
+            button = html.Component(
+                mark_safe(f'<button class="material-icons mdc-icon-button" ryzom-id="308bade28a8c11ebad3800e18cb957e9" style="color: {getattr(view, "color", "")}; --mdc-ripple-fg-size:28px; --mdc-ripple-fg-scale:1.7142857142857142; --mdc-ripple-left:10px; --mdc-ripple-top:10px;">{getattr(view, "icon", "")}</button>'),  # noqa
+                title=view.title.capitalize(),
+                href=view.url + '?_next=' + request.path_info,
+                style='text-decoration: none',
+                tag='a',
+            )
+            if getattr(view, 'controller', None) == 'modal':
+                button.attrs.up_modal = '.main-inner'
+            else:
+                button.attrs['up-target'] = html.A.attrs['up-target']
+            buttons.append(button)
+        return html.Div(*buttons)
+
     def get_model(self):
         return None
 
@@ -358,3 +388,65 @@ class Router(object):
 
     def get_allowed_groups(self):
         return []
+
+    def get_swagger_model_name(self, request):
+        return self.model.__name__
+
+    def get_swagger_model_definition(self, request):
+        result = dict(
+            properties={},
+            type='object',
+            # TODO:
+            # 'required': ['name', 'photoUrls'],
+        )
+        for field in self.model._meta.fields:
+            res = getattr(self, 'swagger_field_{field.name}_definition', None)
+            if res:
+                result['properties'][field.name] = res
+                continue
+
+            field_def = dict()
+            # relate to one model
+            # field_def = {'$ref': '#/definitions/' + field.model.__name__}
+
+            int_fields = (
+                models.IntegerField,
+                models.PositiveIntegerField,
+            )
+            if isinstance(field, int_fields):
+                field_def['type'] = 'integer'
+            elif isinstance(field, models.BooleanField):
+                field_def['type'] = 'boolean'
+            elif isinstance(field, models.JSONField):
+                field_def['type'] = 'object'
+            else:
+                field_def['type'] = 'string'
+
+            result['properties'][field.name] = field_def
+        return result
+
+    def get_json_fields(self):
+        return [
+            f.name for f in self.model._meta.fields
+        ]
+
+    def get_FIELD_json(self, obj, field):
+        value = getattr(obj, field)
+        if callable(value):
+            value = value()
+        if type(value) in self.registry:
+            value = self.registry[type(value)].serialize(value)
+        elif value and not isinstance(value, (str, int, float, bool)):
+            value = str(value)
+        return value
+
+    def serialize(self, obj, fields=None):
+        fields = fields or self.json_fields
+        return {
+            field: getattr(
+                self,
+                f'get_{field}_json',
+                self.get_FIELD_json,
+            )(obj, field)
+            for field in self.json_fields
+        }
