@@ -10,22 +10,20 @@ Crudlfa+ takes views further than Django and are expected to:
 
 import logging
 
-from crudlfap import shortcuts as crudlfap
-
 from django import http
-from django.contrib import auth
-from django.contrib import messages
+from django.contrib import auth, messages
 from django.contrib.auth.forms import PasswordChangeForm, SetPasswordForm
 from django.utils.translation import ugettext_lazy as _
 
+from crudlfap import shortcuts as crudlfap
 
 logger = logging.getLogger()
 
 
 class PasswordView(crudlfap.UpdateView):
     slug = 'password'
-    material_icon = 'vpn_key'
-    color = 'purple darken-4'
+    icon = 'vpn_key'
+    color = 'purple'
     controller = 'modal'
     action = 'click->modal#open'
 
@@ -49,10 +47,16 @@ class PasswordView(crudlfap.UpdateView):
 class BecomeUser(crudlfap.ObjectView):
     urlname = 'su'
     menus = ['object']
-    material_icon = 'attach_money'
-    color = 'pink darken-4'
+    icon = 'accessibility_new'
+    color = 'purple'
     link_attributes = {'data-noprefetch': 'true'}
 
+    def become(self):
+        """
+        Implement this method to override the default become logic.
+        """
+        auth.login(self.request, self.object, backend=self.backend)
+
     def get_title_menu(self):
         return _('become').capitalize()
 
@@ -60,7 +64,7 @@ class BecomeUser(crudlfap.ObjectView):
         user = super().get_object()
 
         if user:
-            user.backend = 'django.contrib.auth.backends.ModelBackend'
+            user.backend = self.backend
         else:
             messages.error(
                 self.request,
@@ -71,19 +75,21 @@ class BecomeUser(crudlfap.ObjectView):
 
     def get(self, request, *a, **k):
         logger.info('BecomeUser by {}'.format(self.request.user))
-        become_user_realname = str(self.request.user)
-        become_user = request.session.get('become_user', request.user.pk)
-        auth.login(request, self.object)
-        request.session.setdefault(
-            'become_user_realname',
-            become_user_realname
-        )
-        request.session['become_user'] = become_user
+        request_user = request.user
+        self.request = request
+        result = self.become()
+        if 'become_user' not in request.session:
+            request.session['become_user_realname'] = str(request_user)
+            request.session['become_user'] = str(request_user.pk)
         messages.info(
             request,
             _('Switched to user %s') % request.user
         )
-        return http.HttpResponseRedirect('/' + self.router.registry.urlpath)
+        return result if result else http.HttpResponseRedirect(
+            '/' + self.router.registry.urlpath)
+
+    def get_backend(self):
+        return 'django.contrib.auth.backends.ModelBackend'
 
 
 class Become(crudlfap.View):
@@ -92,9 +98,12 @@ class Become(crudlfap.View):
     def has_perm(self):
         return 'become_user' in self.request.session
 
+    def get_backend(self):
+        return 'django.contrib.auth.backends.ModelBackend'
+
     def get_object(self):
         user = self.model.objects.get(pk=self.request.session['become_user'])
-        user.backend = 'django.contrib.auth.backends.ModelBackend'
+        user.backend = self.backend
         return user
 
     def get(self, request, *a, **k):
@@ -104,19 +113,10 @@ class Become(crudlfap.View):
                 'No become_user in session {}'.format(self.request.user))
             return http.HttpResponseNotFound()
 
-        if 'become_user' in request.session:
-            user = self.get_object()
-            auth.login(request, user)
-            messages.info(
-                request,
-                _('Switched back to your user %s') % user
-            )
-            if 'become_user' in request.session.keys():
-                del request.session['become_user']
-        else:
-            messages.warning(
-                request,
-                _('You are still superuser %s') % self.request.user
-            )
-
+        user = self.get_object()
+        auth.login(request, user)
+        messages.info(
+            request,
+            _('Switched back to your user %s') % user
+        )
         return http.HttpResponseRedirect('/' + self.registry.urlpath)
